>From e6b76c0b50cba85e9319f2d622e8bc8a28084c6e Mon Sep 17 00:00:00 2001
From: Erik Van Grunderbeeck <erik@arawix.com>
Date: Tue, 22 Mar 2011 11:32:13 +0100
Subject: [PATCH 5/5] Do direct load of Java VM through LoadLibrary on Win32. Loading through dl_sym() will destroy the function pointers on a 64 bit VM on Win32 (not sure for *Nix)
 Add correct prototype for VM for win32 if loading through a dll. Otherwise stack gets messed up, and loading VM will crash libbluray 1/2 times.
 Add option to start direct connection to Eclipse to debug the Java classes of libbluray.

Signed-off-by: Erik Van Grunderbeeck <erik@arawix.com>
---
 src/libbluray/bdj/bdj.c |   41 +++++++++++++++++++++++++++++++++--------
 1 files changed, 33 insertions(+), 8 deletions(-)

diff --git a/src/libbluray/bdj/bdj.c b/src/libbluray/bdj/bdj.c
index ce787f1..42094e6 100644
--- a/src/libbluray/bdj/bdj.c
+++ b/src/libbluray/bdj/bdj.c
@@ -17,6 +17,10 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#ifdef _WIN32
+#include <windows.h>
+#endif	//	#ifdef _WIN32
+
 #include "bdj.h"
 
 #if HAVE_CONFIG_H
@@ -33,7 +37,14 @@
 #include <stdlib.h>
 #include <string.h>
 
-typedef jint (*JNICALL fptr_JNI_CreateJavaVM)(JavaVM **pvm, void **penv,void *args);
+/*
+ Turns this define on to debug the libbluray java classes with Eclipse. 
+ To debug, load the jar file or project with Eclipse. Start debugging by waiting on an external connection
+*/
+//#define DEBUG_CONNECTION
+
+//	signature for VM
+typedef jint (JNICALL * fptr_JNI_CreateJavaVM) (JavaVM **pvm, void **penv,void *args);
 
 int start_xlet(JNIEnv* env, const char* path, jobject bdjo, BDJAVA* bdjava);
 void* load_jvm();
@@ -59,20 +70,30 @@ BDJAVA* bdj_open(const char *path, const char* start, void* bd, void* registers)
     if (classpath == NULL)
         classpath = BDJ_CLASSPATH;
 
+    JavaVMOption* option = malloc(sizeof(JavaVMOption) * 3);
+
     // determine classpath
     char* classpath_opt = str_printf("-Djava.class.path=%s", classpath);
-
-    JavaVMOption* option = malloc(sizeof(JavaVMOption) * 1);
     option[0].optionString = classpath_opt;
+    args.nOptions = 1;
+#ifdef DEBUG_CONNECTION
+    //	do debug options
+    option[1].optionString = "-Xdebug";
+    //	do debug options
+    option[2].optionString = "-Xrunjdwp:transport=dt_socket,address=7777,server=y,suspend=y";
+    //	have 3 now
+    args.nOptions = 3;
+#endif	//	#ifdef DEBUG_CONNECTION
 
     args.version = JNI_VERSION_1_6;
-    args.nOptions = 1;
     args.options = option;
     args.ignoreUnrecognized = JNI_FALSE; // don't ignore unrecognized options
 
-    fptr_JNI_CreateJavaVM JNI_CreateJavaVM_fp = (fptr_JNI_CreateJavaVM)dl_dlsym(jvm_lib,
-            "JNI_CreateJavaVM");
-
+#ifdef WIN32
+    fptr_JNI_CreateJavaVM JNI_CreateJavaVM_fp = (fptr_JNI_CreateJavaVM)GetProcAddress(jvm_lib, "JNI_CreateJavaVM");
+#else	//	#ifdef WIN32
+    fptr_JNI_CreateJavaVM JNI_CreateJavaVM_fp = (fptr_JNI_CreateJavaVM)dl_dlsym(jvm_lib, "JNI_CreateJavaVM");
+#endif	//	#ifdef WIN32
     if (JNI_CreateJavaVM_fp == NULL) {
         free(bdjava);
         free(option);
@@ -81,10 +102,14 @@ BDJAVA* bdj_open(const char *path, const char* start, void* bd, void* registers)
         return NULL;
     }
 
-    int result = JNI_CreateJavaVM_fp(&bdjava->jvm, (void**) &bdjava->env, &args);
+    //	do the call
+    int result = (*JNI_CreateJavaVM_fp)(&bdjava->jvm, (void**) &bdjava->env, &args);
+
+    //	free what we dont need
     free(option);
     free(classpath_opt);
 
+    //	check
     if (result != JNI_OK || !bdjava->env) {
         free(bdjava);
         BD_DEBUG(DBG_BDJ | DBG_CRIT, "Failed to create new Java VM.\n");
-- 
1.7.0.2.msysgit.0

